# Vue File Uploader

一个支持 Vue 2.x 和 Vue 3.x 的文件上传组件，支持拖拽上传、多文件上传、进度显示等功能。

## 更新日志

### 1.4.0
- ✨ 优化文件上传性能
- 🔧 改进错误处理机制
- 🎨 优化组件样式和交互体验
- 📦 优化包体积
- 🐛 修复已知问题

### 1.3.0
- ✨ 优化 Vue 3 支持，移除不必要的 composition-api 依赖
- 🔧 改进类型定义，更好的 TypeScript 支持
- 📝 更新文档，添加 Vue 3 setup 语法糖示例
- 🐛 修复 Vue 3 环境下的类型导入问题
- 🎨 优化组件样式和交互体验
- 🚀 提升构建性能和包体积优化

### 1.2.0
- ✨ 新增 `clearFiles` 方法用于清空文件列表
- ✨ 新增 `tip` 插槽用于自定义提示信息
- ✨ 新增 `progress` 事件用于监听上传进度
- 🐛 修复 Vue 2.x 环境下的类型定义问题
- 🐛 修复文件大小限制的错误处理
- 📦 优化 ESM 和 CommonJS 模块导出
- 📝 完善 TypeScript 类型定义

### 1.0.0
- 🎯 初始版本发布
- ✨ 支持 Vue 2.x 和 Vue 3.x
- ✨ 支持点击和拖拽上传
- ✨ 支持单文件和多文件上传
- ✨ 支持文件类型和大小限制
- ✨ 支持自定义上传区域
- ✨ 支持上传进度显示
- 💪 使用 TypeScript 编写，提供完整的类型定义

## 特性

- 🎯 支持 Vue 2.x 和 Vue 3.x
- 🚀 支持点击和拖拽上传
- 📦 支持单文件和多文件上传
- 📊 实时显示上传进度
- 🎨 美观的 UI 和动画效果
- 🔧 高度可定制的界面
- 💪 使用 TypeScript 编写，提供完整的类型定义
- 📦 支持 ESM 和 CommonJS 两种模块规范

## 安装

```bash
# Vue 3.x
npm install @saberlayer/vue-file-uploader

# Vue 2.x - 需要同时安装 composition-api
npm install @saberlayer/vue-file-uploader @vue/composition-api
```

## 使用方法

### 基础用法

```vue
<template>
  <div class="upload-demo">
    <file-uploader
      v-model:fileList="fileList"
      action="https://your-upload-url.com/upload"
      :headers="headers"
      :data="formData"
      :multiple="true"
      :max-size="5 * 1024 * 1024"
      :max-count="5"
      accept=".jpg,.png,.pdf"
      @change="handleChange"
      @success="handleSuccess"
      @error="handleError"
      @exceed="handleExceed"
      @progress="handleProgress"
    >
      <template #default>
        <div class="upload-trigger">
          <i class="upload-icon">+</i>
          <div class="upload-text">点击或拖拽文件上传</div>
          <div class="upload-hint">支持 jpg、png、pdf 格式，单个文件不超过 5MB</div>
        </div>
      </template>
    </file-uploader>
    
    <!-- 上传进度展示 -->
    <div v-if="currentFile" class="upload-progress">
      <div class="progress-info">
        <span class="filename">{{ currentFile.name }}</span>
        <span class="percentage">{{ currentProgress }}%</span>
      </div>
      <div class="progress-bar">
        <div class="progress-inner" :style="{ width: currentProgress + '%' }"></div>
      </div>
    </div>
  </div>
</template>

<script setup lang="ts">
import { ref, reactive } from 'vue'
import type { UploadFile } from '@saberlayer/vue-file-uploader'

// 文件列表
const fileList = ref<UploadFile[]>([])

// 当前上传文件信息
const currentFile = ref<UploadFile | null>(null)
const currentProgress = ref(0)

// 上传请求头
const headers = reactive({
  'Authorization': 'Bearer your-token-here'
})

// 上传时附带的额外参数
const formData = reactive({
  userId: 'user123',
  projectId: 'project456'
})

// 文件状态改变
const handleChange = ({ file, fileList }: { file: UploadFile; fileList: UploadFile[] }) => {
  console.log('文件变化：', file, fileList)
  // 可以在这里对文件列表进行过滤或处理
  const filteredList = fileList.filter(f => {
    // 例如：过滤掉上传失败的文件
    return f.status !== 'error'
  })
  fileList.value = filteredList
}

// 上传进度
const handleProgress = (file: UploadFile, percentage: number) => {
  currentFile.value = file
  currentProgress.value = Math.floor(percentage)
}

// 上传成功
const handleSuccess = (file: UploadFile, fileList: UploadFile[]) => {
  console.log('上传成功：', file)
  // 重置进度显示
  if (currentFile.value?.uid === file.uid) {
    setTimeout(() => {
      currentFile.value = null
      currentProgress.value = 0
    }, 1000)
  }
  
  // 可以在这里处理服务器返回的数据
  if (file.response) {
    // 例如：更新文件URL
    file.url = file.response.url
  }
}

// 上传失败
const handleError = (error: Error, file: UploadFile) => {
  console.error('上传失败：', error)
  // 显示错误提示
  alert(`文件 ${file.name} 上传失败：${error.message}`)
}

// 超出限制
const handleExceed = (files: File[]) => {
  console.warn('超出文件限制：', files)
  alert(`最多只能上传 5 个文件，每个文件不超过 5MB`)
}
</script>

<style scoped>
.upload-demo {
  width: 100%;
  max-width: 600px;
  margin: 0 auto;
}

.upload-trigger {
  padding: 24px;
  border: 2px dashed #e0e0e0;
  border-radius: 8px;
  text-align: center;
  cursor: pointer;
  transition: border-color 0.3s;
}

.upload-trigger:hover {
  border-color: #1890ff;
}

.upload-icon {
  font-size: 28px;
  color: #999;
}

.upload-text {
  margin-top: 8px;
  font-size: 16px;
  color: #666;
}

.upload-hint {
  margin-top: 4px;
  font-size: 14px;
  color: #999;
}

.upload-progress {
  margin-top: 16px;
  padding: 12px;
  background: #f5f5f5;
  border-radius: 4px;
}

.progress-info {
  display: flex;
  justify-content: space-between;
  margin-bottom: 8px;
}

.filename {
  color: #333;
  font-size: 14px;
}

.percentage {
  color: #1890ff;
  font-size: 14px;
}

.progress-bar {
  height: 4px;
  background: #e0e0e0;
  border-radius: 2px;
  overflow: hidden;
}

.progress-inner {
  height: 100%;
  background: #1890ff;
  transition: width 0.3s ease;
}
</style>
```

### 自定义上传实现

如果需要自定义上传逻辑，可以通过 `:before-upload` 和 `:http-request` 属性来实现：

```vue
<template>
  <file-uploader
    ref="uploader"
    v-model:fileList="fileList"
    action="https://your-upload-url.com/upload"
    :before-upload="handleBeforeUpload"
    :http-request="customUpload"
    :auto-upload="false"
  >
    <template #default>
      <div class="custom-upload">
        <button class="select-btn" @click="$refs.uploader.handleClick()">
          选择文件
        </button>
        <p class="drag-tip">或将文件拖拽到此处</p>
      </div>
    </template>
  </file-uploader>
  
  <!-- 上传控制按钮 -->
  <div class="upload-controls">
    <button class="upload-btn" @click="handleUpload">开始上传</button>
    <button class="clear-btn" @click="handleClear">清空列表</button>
  </div>
</template>

<script setup lang="ts">
import { ref } from 'vue'
import type { UploadFile } from '@saberlayer/vue-file-uploader'
import axios from 'axios'

const uploader = ref()
const fileList = ref<UploadFile[]>([])

// 上传前的处理
const handleBeforeUpload = (file: File) => {
  // 返回 false 或 Promise.reject() 会阻止上传
  if (file.size > 10 * 1024 * 1024) {
    alert('文件大小不能超过 10MB')
    return false
  }
  
  // 可以返回处理后的文件
  // 例如：压缩图片
  if (file.type.startsWith('image/')) {
    return compressImage(file)
  }
  
  return true
}

// 自定义上传实现
const customUpload = async ({ file, onProgress, onSuccess, onError }: {
  file: File,
  onProgress: (progress: number) => void,
  onSuccess: (response: any) => void,
  onError: (error: Error) => void
}) => {
  const formData = new FormData()
  formData.append('file', file)
  
  try {
    const response = await axios.post('https://your-upload-url.com/upload', formData, {
      headers: {
        'Content-Type': 'multipart/form-data',
        'Authorization': 'Bearer your-token-here'
      },
      onUploadProgress: (progressEvent) => {
        const progress = (progressEvent.loaded / (progressEvent.total || 0)) * 100
        onProgress(Math.floor(progress))
      }
    })
    
    onSuccess(response.data)
  } catch (error) {
    onError(error as Error)
  }
}

// 压缩图片
const compressImage = (file: File): Promise<File> => {
  return new Promise((resolve, reject) => {
    const reader = new FileReader()
    reader.readAsDataURL(file)
    reader.onload = (e) => {
      const img = new Image()
      img.src = e.target?.result as string
      img.onload = () => {
        const canvas = document.createElement('canvas')
        const ctx = canvas.getContext('2d')
        
        // 设置压缩后的尺寸
        let width = img.width
        let height = img.height
        const maxSize = 1024
        
        if (width > height && width > maxSize) {
          height = Math.round((height * maxSize) / width)
          width = maxSize
        } else if (height > maxSize) {
          width = Math.round((width * maxSize) / height)
          height = maxSize
        }
        
        canvas.width = width
        canvas.height = height
        
        ctx?.drawImage(img, 0, 0, width, height)
        
        // 转换为 Blob
        canvas.toBlob((blob) => {
          if (blob) {
            const newFile = new File([blob], file.name, {
              type: 'image/jpeg',
              lastModified: Date.now()
            })
            resolve(newFile)
          } else {
            reject(new Error('图片压缩失败'))
          }
        }, 'image/jpeg', 0.8)
      }
    }
    reader.onerror = (error) => reject(error)
  })
}

// 开始上传
const handleUpload = () => {
  uploader.value?.submit()
}

// 清空列表
const handleClear = () => {
  uploader.value?.clearFiles()
}
</script>

<style scoped>
.custom-upload {
  padding: 24px;
  border: 2px dashed #e0e0e0;
  border-radius: 8px;
  text-align: center;
}

.select-btn {
  padding: 8px 16px;
  background: #1890ff;
  color: white;
  border: none;
  border-radius: 4px;
  cursor: pointer;
}

.drag-tip {
  margin-top: 8px;
  color: #666;
}

.upload-controls {
  margin-top: 16px;
  display: flex;
  gap: 12px;
}

.upload-btn,
.clear-btn {
  padding: 8px 24px;
  border: none;
  border-radius: 4px;
  cursor: pointer;
}

.upload-btn {
  background: #52c41a;
  color: white;
}

.clear-btn {
  background: #ff4d4f;
  color: white;
}
</style>
```

## API

### Props

| 参数 | 说明 | 类型 | 默认值 |
|------|------|------|--------|
| action | 上传的地址（必填） | string | - |
| multiple | 是否支持多文件上传 | boolean | false |
| accept | 接受的文件类型 | string | - |
| maxSize | 文件大小限制（字节） | number | 0 |
| maxCount | 文件数量限制 | number | 0 |
| headers | 上传请求头 | Record<string, string> | {} |
| data | 上传时附带的额外参数 | Record<string, any> | {} |
| autoUpload | 是否在选择文件后自动上传 | boolean | true |
| fileList | 已上传的文件列表 | UploadFile[] | [] |
| disabled | 是否禁用上传 | boolean | false |
| drag | 是否启用拖拽上传 | boolean | false |

### Events

| 事件名 | 说明 | 回调参数 |
|--------|------|----------|
| change | 文件状态改变时触发 | { file: UploadFile; fileList: UploadFile[] } |
| success | 文件上传成功时触发 | (file: UploadFile, fileList: UploadFile[]) |
| error | 文件上传失败时触发 | (error: Error, file: UploadFile) |
| exceed | 超出文件数量或大小限制时触发 | (files: File[]) |
| progress | 上传进度变化时触发 | (file: UploadFile, percentage: number) |

### Methods

| 方法名 | 说明 | 参数 |
|--------|------|------|
| submit | 手动上传文件 | - |
| handleClick | 手动触发文件选择 | - |
| clearFiles | 清空文件列表 | - |

### Slots

| 插槽名 | 说明 |
|--------|------|
| default | 自定义上传区域的内容 |
| tip | 自定义提示信息 |

## 浏览器支持

- Chrome >= 51
- Firefox >= 53
- Safari >= 10
- Edge >= 79

## 开发

```bash
# 安装依赖
npm install

# 开发模式
npm run dev

# 构建 Vue 2.x 版本
npm run build:v2

# 构建 Vue 3.x 版本
npm run build:v3

# 构建所有版本
npm run build

# 运行示例
npm run demo

# 运行测试
npm run test
```

## 常见问题

### 1. Vue 2.x 环境下报错 "Composition API is not available"

确保已安装并注册 `@vue/composition-api`：

```javascript
import Vue from 'vue'
import VueCompositionAPI from '@vue/composition-api'

Vue.use(VueCompositionAPI)
```

### 2. 文件上传失败

检查以下几点：
- 上传地址 `action` 是否正确
- 服务器是否支持跨域请求
- 文件大小是否超出限制
- 网络连接是否正常

### 3. IE 11 兼容性问题

需要添加以下 polyfill：
- `core-js`
- `regenerator-runtime`

## License

MIT 